from __future__ import print_function
import sys
import time
import six
import argparse
import json
import re
from .User import User
from .UserFormatError import UserFormatError

class HashProcessor(object):
    def __init__(self):
        self.description = "Hashes entries from a file"

    def setup_argparse(self, parser):
        parser.add_argument("file", type=str, \
                help="email address or path to file containing email addresses to hash.")
        parser.add_argument("--type", nargs="+", action="store", choices=["md5","sha1","sha256"], required=False, \
                default=[ "md5", "sha1", "sha256" ], help='List of hash algorithms separated by whitespaces (md5 sha1 sha256). Default = md5, sha1, sha256')

    def processLine(self, line, args):
        line = line.strip()
        try:
            user = User(line.strip())
            if(user.user_type=="email"):
                hashes = user.hashes
                if "md5" in args.type:
                    print(hashes[0])
                if "sha1" in args.type:
                    print(hashes[1])
                if "sha256" in args.type:
                    print(hashes[2])
            else:
                print("skipping - " + line, file=sys.stderr)
        except UserFormatError:
            print("skipping - " + line, file=sys.stderr)

    def execute(self, args):
        # Read the config file
        try:
            with open(args.file, "r") as hashlist:
                for line in hashlist:
                    self.processLine(line, args)
        except FileNotFoundError:
            self.processLine(args.file, args)



from .DSRProcessor import DSRProcessor
from .RequestFactory import RequestFactory

class OptoutProcessor(DSRProcessor):
    def __init__(self):
        DSRProcessor.__init__(self, "OBJECT", True)
        self.description = "submits an optout request for a user."

    def construct_request(self, identifiers):
        return RequestFactory.optout(self.config['domain_name'], self.config['key_id'], self.config['scope'], identifiers=identifiers, callback_url=self.config.get('callback_url'), request_id=self.config.get('request_id'))



from .DSRProcessor import DSRProcessor
from .RequestFactory import RequestFactory

class PingProcessor(DSRProcessor):
    def __init__(self):
        DSRProcessor.__init__(self, "PING", False)
        self.description = "submits a ping request to validate keys and connection."

    def construct_request(self):
        return RequestFactory.ping(self.config['domain_name'], self.config['key_id'], self.config.get('callback_url'), self.config.get('request_id'))

    def execute(self, args):
        self._initialize(args)
        request = self.construct_request()
        if self.config["verbose"]:
            print("Request: {}".format(request))
        response = self.api_client.submit(request)
        self.print_response(response)



from __future__ import print_function
import sys
import argparse
import li_privacy
import signal

def signal_handler(sig, frame):
    print('Interrupted by Ctrl+C!')
    sys.exit(0)

def main(name=None):
    signal.signal(signal.SIGINT, signal_handler)
    parser = argparse.ArgumentParser(
            description="Interact with the LiveIntent Privacy API", \
            epilog="For API documentation, see https://link.liveintent.com/privacy-api")
    parser.add_argument('--version', action='version', \
            version='%(prog)s v.{version}'.format(version=li_privacy.__version__))

    subparsers = parser.add_subparsers(title="actions", dest='command')

    actions = {
        "init": li_privacy.InitProcessor(),
        "delete":  li_privacy.DeleteProcessor(),
        "optout": li_privacy.OptoutProcessor(),
        "ping": li_privacy.PingProcessor(),
        "hash": li_privacy.HashProcessor()
    }
    for action in actions:
        processor = actions[action]
        sub_parser = subparsers.add_parser(action, help=processor.description)
        processor.setup_argparse(sub_parser)

    args = parser.parse_args(sys.argv[1:])
    try:
        func = actions[args.command]
    except (AttributeError, KeyError):
        parser.print_help()
        return 1
    func.execute(args)

if __name__ == "__main__":
    main()



import re
import hashlib
from .UserFormatError import UserFormatError

class User(object):
    # Ugly regex courtesy of https://emailregex.com/
    EMAIL_PATTERN = """^(?:[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*|"(?:[\x01-\x08\x0b\x0c\x0e-\x1f\x21\x23-\x5b\x5d-\x7f]|\\[\x01-\x09\x0b\x0c\x0e-\x7f])*")@(?:(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?|\[(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?|[a-z0-9-]*[a-z0-9]:(?:[\x01-\x08\x0b\x0c\x0e-\x1f\x21-\x5a\x53-\x7f]|\\[\x01-\x09\x0b\x0c\x0e-\x7f])+)\])$"""

    # 32, 40, or 64 lower-case hexadecimal characters
    HASH_PATTERN = "^[a-f0-9]{32}([a-f0-9]{8}([a-f0-9]{24})?)?$"

    def __init__(self, string):
        string = string.strip().lower()
        if re.match(self.EMAIL_PATTERN, string):
            self.user_type = "email"
            self.hashes = self.hash_email(string)
        elif re.match(self.HASH_PATTERN, string):
            self.user_type = "hash"
            self.hashes = [ string ]
        else:
            raise UserFormatError("Input does not appear to be a valid email or hash")

    def hash_email(self, email_address):
        bytes = email_address.encode('utf-8')
        return [
            hashlib.md5(bytes).hexdigest(),
            hashlib.sha1(bytes).hexdigest(),
            hashlib.sha256(bytes).hexdigest()
        ]

    def json(self):
        return [
                { "type": "EMAIL_HASH", "values": self.hashes }
        ]



from __future__ import print_function
import json
import time

class Request(object):
    """Represents a request payload to the li-privacy API"""

    def __init__(self, operation, path, domain_name, key_id, callback_url, scope=None, identifiers=None, request_id=None, iat=int(time.time()), expires_in=None):
        self.operation = operation
        self.path = path
        self.domain_name = domain_name
        self.key_id = key_id
        self.callback_url = callback_url
        self.scope = scope
        self.identifiers = identifiers
        self.request_id = request_id or str(iat)
        self.iat = iat

        self.expires_in = expires_in or 3600 # 1 Hour

    def json(self):
        payload = { 
            "iss": "CN={}".format(self.domain_name),
            "aud": "privacy-api.liveintent.com",
            "cnf": {
                "kid": self.key_id
            },
            "jti": self.request_id,
            "iat": self.iat,
            "exp": self.iat + self.expires_in,
            "dsr": {
                "type": self.operation
            }
        }
        # Optional fields
        if self.scope is not None:
            payload['dsr']['scope'] = self.scope
        if self.identifiers is not None:
            payload['dsr']['identifiers'] = self.identifiers.json()
        if self.callback_url is not None:
            payload['dsr']['target'] = self.callback_url
        return payload

    def __repr__(self):
        return json.dumps(self.json())



__title__ = 'li_privacy'
__description__ = 'CLI and sample application to interact with the LiveIntent privacy API'
__url__ = 'https://link.liveintent.com/privacy-api'
__version__ = '1.3.1'
__author__ = 'Phillip Markert'
__author_email__ = 'phillip@liveintent.com'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2020 LiveIntent'



import jwt
import json
import requests

class ApiClient(object):
    def __init__(self, hostname, rsa_key, verbose):
        self.verbose = verbose
        self.rsa_key = rsa_key
        self.hostname = hostname

    def encode_jwt(self, payload):
        return jwt.encode(payload, self.rsa_key, algorithm="RS256").decode('utf-8')

    def wrap_jwt(self, jwt):
        return { "jwt": jwt }

    def send_request(self, path, body):
        url = "https://{}/{}".format(self.hostname, path)
        if(self.verbose):
            print("Curl equivalent:\n curl -X POST -H 'Content-type: application/json' -d '{data}' '{url}'".format(data=json.dumps(body), url=url))
        return requests.post(url, json=body)

    def submit(self, request):
        jwt = self.encode_jwt(request.json()) 
        body = self.wrap_jwt(jwt)
        return self.send_request(request.path, body)



from __future__ import print_function
import sys
import six
import argparse
import json
import re
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.backends import default_backend
from .DSRProcessor import DSRProcessor

class InitProcessor(DSRProcessor):
    def __init__(self):
        DSRProcessor.__init__(self, "PING", False)
        self.description = "sets account configuration and generates keys"

    def setup_argparse(self, parser):
        parser.add_argument("--config", type=str, default="config.json", \
                help="path to configuration file (defaults to config.json)")
        parser.add_argument("--domain_name", type=str, \
                help="your domain name. Use 'dailyplanet.com' to generate example keys and config")
        parser.add_argument("--key_id", type=str, default="key1", \
                help="the signing key identifier")
        parser.add_argument("--signing_key", type=str, \
                help="path to RSA-256 private signing key file. Will generate a new key-pair if missing.")

    def generate_key(self, signing_key):
        try:
            # Attempt to read existing key
            with open(signing_key) as f:
                private_key = f.read()
            print("Using existing key in " + signing_key)

        except IOError:
            # generate new private/public key pair
            key = rsa.generate_private_key(backend=default_backend(), public_exponent=65537, \
                key_size=2048)

            public_key = key.public_key().public_bytes(serialization.Encoding.PEM, \
                serialization.PublicFormat.SubjectPublicKeyInfo)

            pem = key.private_bytes(encoding=serialization.Encoding.PEM,
                format=serialization.PrivateFormat.TraditionalOpenSSL,
                encryption_algorithm=serialization.NoEncryption())

            private_key = pem.decode('utf-8')
            with open(signing_key, "w+") as private_key_file:
                private_key_file.write(private_key)

            public_key = public_key.decode('utf-8')
            with open(signing_key + ".pub", "w+") as public_key_file:
                public_key_file.write(public_key)

            print("Generated new keys in " + signing_key + " and " + signing_key + ".pub")

        return private_key

    def print_provisioning_notice(self, args, config):
        print("")
        print("To provision your keys, please email the following files to privacy@liveintent.com:")
        print("\t{}".format(args.config))
        print("\t{}.pub".format(config['signing_key']))
        print("")

    def generate_example_config(self, args, config):
        print("Generating example key and configuration")
        args.config = "dailyplanet.json"
        key_id = config['key_id'] = "key1"
        signing_key = config['signing_key'] = config['domain_name'] + ".key"
        config['staging'] = True

        # These example keys are only valid in the staging environment which does not execute live transactions
        # therefore they are safe to embed in this source. These keys are also documented in our API guide
        # and are included here for end-user convenience.
        # For obvious reasons, these keys do not work in the production system. :)

        with open(signing_key, "w+") as private_key:
            private_key.write("""-----BEGIN RSA PRIVATE KEY-----
MIICWwIBAAKBgQCXetR4Wz3YxxEZxArubSXHtkACZ9CIPvc7r9AqmfCR4UM+xG5G
7VMU8KRDZrmEaKUzHWVmRSolDIGPFGXjv+csAzBA2aASI4PkxbeYov7xYFD1lQ4k
TTeg+bj0UaivNOChFUHMWwe5I/sVh7wcwIA1kJfQ15lJOgwBfz5fP8URKwIDAQAB
AoGALj0jQD3xygsx8CCEibUtlCHQtitEX2KBC2oma+qjoZQWd8F0PBhThQ/TxHNF
6+IZk1nEywwPylFf9vHuDDBW+wMg9oErNd6C/KlsVaPth/cQxWU5E5IlaANg5rKA
4VbisjXDkc0H/UXc+Dka4CGwfbdHm7ZylCXgYKNtlLtqcRECQQDFqBe6rz/B3vsU
nxyIxPo6BkrVtKhZHoyy+O69qAfy+VpDBnaA3kUWXwvVpvu1QGkdOYMeKJ1j0jYb
2Ayj3tQpAkEAxDFj5RcLnL3dSVORGUBt3uOOqJc+g2JQMZkbElM+9CZMtvOqghWQ
LrLuJKaqADkjnNNrhwRZ+49ivRBnJdwFMwJAVFN6jDLoSJYRGKMpUVB4UPkORE5m
5F6cOF7rvA5MFeU8FQxU0nYBk6HJMsWi7ZklP0qiHePGAihU3Vw3SFJwwQJATKhb
ttyNTf4lo4wCatJw26EgUaFe7KkSWn7PRBbAx1bbrLSCj/dq8cQ6Jpn0XMf2sUUu
g3/gxNkepG7vTqysXwJAE23D11+RZW4hM92TC8zohTw/jgEXR/klWMWykMuidc+M
t1n7m+87k1K1LL11cfK3X4jbdfv0am1EDOtsPIhvzQ==
-----END RSA PRIVATE KEY-----""")
        with open(signing_key + ".pub", "w+") as public_key:
            public_key.write("""-----BEGIN PUBLIC KEY-----
MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQCXetR4Wz3YxxEZxArubSXHtkAC
Z9CIPvc7r9AqmfCR4UM+xG5G7VMU8KRDZrmEaKUzHWVmRSolDIGPFGXjv+csAzBA
2aASI4PkxbeYov7xYFD1lQ4kTTeg+bj0UaivNOChFUHMWwe5I/sVh7wcwIA1kJfQ
15lJOgwBfz5fP8URKwIDAQAB
-----END PUBLIC KEY-----""")
        print("Saved example keys in " + signing_key + " and " + signing_key + ".pub")
        print()
        print("To use these example keys, add --config " + args.config + " to your command.")
        print()

    def get_domain_name(self, args, config):
        domain_name = args.domain_name or config.get('domain_name','')
        while True:
            if domain_name:
                default_prompt = "({}) ".format(domain_name)
            else:
                default_prompt = ""
            config['domain_name'] = six.moves.input("Your domain name: " + default_prompt) or domain_name
            if not re.match("(?:[a-z0-9](?:[a-z0-9-]{0,61}[a-z0-9])?\.)+[a-z0-9][a-z0-9-]{0,61}[a-z0-9]", config['domain_name']):
                print("Please enter your DNS name only (i.e. publisher.com)")
            else:
                return domain_name

    def execute(self, args):
        # Read the config file
        try:
            with open(args.config) as config_json:
                config = json.load(config_json)
            print("Using existing config: %s" % args.config)
        except IOError:
            config = {}
            print("Creating new config: %s" % args.config)
        print()

        domain_name = self.get_domain_name(args, config)
        config['callback_url'] = config.get("callback_url","")

        example_configuration = config['domain_name']=="dailyplanet.com"
        if example_configuration:
            self.generate_example_config(args, config)
        else:
            key_id = args.key_id or config.get('key_id','')
            config['key_id'] = six.moves.input("Key Identifier: (%s) " % key_id) or key_id

            signing_key = args.signing_key or config.get('signing_key', config['domain_name'] + ".key")
            config['signing_key'] = six.moves.input("Private RSA signing key file: ({}) ".format(signing_key)) or signing_key

            rsa_key = self.generate_key(config['signing_key'])

        # Write config file
        with open(args.config, "w+") as config_file:
            json.dump(config, config_file, indent=2)
        print("Configuration written to {}".format(args.config))
        if not example_configuration:
            self.print_provisioning_notice(args, config)



from .Request import Request

class RequestFactory(object):
    @staticmethod
    def ping(domain_name, key_id, callback_url=None, request_id=None):
        """Factory method to create a new ping request with the specified domain_name, key_id, and optional callback_url and request_id"""
        return Request("PING", "ping", domain_name, key_id, callback_url, identifiers=None, request_id=request_id)

    @staticmethod
    def optout(domain_name, key_id, scope, identifiers, callback_url=None, request_id=None):
        # identifiers is identifiers object
        return Request("OBJECT", "dsr", domain_name, key_id, callback_url, scope, identifiers, request_id=request_id)

    @staticmethod
    def delete(domain_name, key_id, scope, identifiers, callback_url=None, request_id=None):
        # Validate identifiers is object
        return Request("ERASURE", "dsr", domain_name, key_id, callback_url, scope, identifiers, request_id=request_id)

    @staticmethod
    def access(domain_name, key_id, scope, identifiers, callback_url, request_id=None):
        # Validate callback is string
        return Request("ACCESS", "dsr", domain_name, key_id, callback_url, scope, identifiers, request_id=request_id)



from __future__ import print_function
import sys
import argparse
import json
import time
import os.path
from .UserFormatError import UserFormatError
from .ApiClient import ApiClient
from .User import User

class DSRProcessor(object):
    def __init__(self, operation, requires_user=True):
        # Setup properties
        self.operation = operation
        self.requires_user = requires_user
        self.config = {}
        self.api_client = None

    def setup_argparse(self, parser):
        parser.add_argument("--config", type=str, default="config.json", \
                help="path to configuration file (Defaults to config.json)")
        parser.add_argument("--scope", type=str, \
                choices=["US_PRIVACY","EU_PRIVACY"], default="US_PRIVACY", \
                help="jurisdiction under which the the request is submitted. (Defaults to US_PRIVACY)")
        parser.add_argument("--callback_url", type=str, \
                help="callback url to be invoked.")
        parser.add_argument("--verbose", "-v", action="store_true", \
                help="enable verbose output")
        parser.add_argument("--staging", action="store_true", \
                help="send to staging environment instead of production.")
        parser.add_argument("--request_id", \
                help="Request ID to be submitted for tracking")
        if self.requires_user:
            parser.add_argument("user", type=str, \
                    help="the email address, hash, or file of users to process")

    def construct_request(self, user):
        """Must be overridden by subclasses"""
        raise Error("Not implemented.")

    def print_headers(self, response):
        print("HTTP/1.1 " + str(response.status_code) + " " + response.reason)
        for key,value in response.headers.items():
            print(key + ": " + value)
        print()

    def print_response(self, response):
        if(not response.ok):
            print("ERROR: API call returned an error.")
            print()
            self.print_headers(response)
        else:
            if self.config['verbose']:
                print("Response received:")
                self.print_headers(response)
                print()
        print(response.text)

    def process_request(self, request):
        if self.config["verbose"]:
            print("Request: {}".format(request))
        return self.api_client.submit(request)

    def process_file(self, filename):
        report_name = "{}.{}.tsv".format(filename,int(time.strftime("%Y%m%d%H%M%S")))
        print("Processing users from file {}".format(filename))
        with open(report_name,"w") as report:
            print("\t".join([ "user", "request_id", "response.ok", "response.text", "timestamp" ]), file=report)
            with open(filename, "r") as hashlist:
                for line in hashlist:
                    line = line.strip()
                    user = User(line)
                    try:
                        request = self.construct_request(user)
                        response = self.process_request(request)
                        print("\t".join([ line, request.request_id, str(response.ok), response.text, str(request.iat) ]), \
                            file=report)
                        print("Processing: {}, success={}".format(line, response.ok))
                    except UserFormatError:
                        print("\t".join([ user, "", "", "Skipped, does not appear to be a valid hash or email", "" ]), \
                            file=report)
                        print("Skipping: '{}', does not appear to be a valid hash or email" \
                            .format(user))
        print("Report saved to {}".format(report_name))

    def process_single(self, user):
        try:
            user = User(user)
            request = self.construct_request(user)
            response = self.process_request(request)
            self.print_response(response)
        except UserFormatError:
            print("ERROR: '{}', does not appear to be a valid hash or email" \
                .format(user))

    def _set_overrideable_parameters(self, args):
        # Overridable parameters
        self.config["staging"] = args.staging or self.config.get("staging", False)
        if args.scope:
            self.config["scope"] = args.scope
        if args.callback_url:
            self.config["callback_url"] = args.callback_url
        if args.request_id:
            self.config["request_id"] = args.request_id

    def _load_config(self, args):
        if args.verbose:
            print("Loading configuration from %s" % args.config)

        # Read the config file
        try:
            with open(args.config) as config_json:
                self.config = json.load(config_json)
        except FileNotFoundError:
            print("Configuration file %s not found.  Run init command to perform initial setup.\n(Are you running from the wrong directory?)" % args.config)
            sys.exit(1)

        self.config['verbose'] = args.verbose
        if self.config['verbose']:
            print("Loaded configuration %s" % json.dumps(self.config, indent=2))

    def _setup_api_client(self, args):
        # Setup api_client with proper environment hostname
        if 'hostname' not in self.config:
            if self.config['staging']:
                self.config["hostname"] = "privacy-test.liadm.com"
            else:
                self.config["hostname"] = "privacy.liadm.com"
        if self.config['verbose']:
            print("Staging=%s, Set API hostname to %s" % (self.config['staging'], self.config["hostname"]))

        # Load signing key
        with open(self.config.get("signing_key")) as key_file:
            rsa_key = key_file.read()

        self.api_client = ApiClient(self.config["hostname"], rsa_key, self.config["verbose"])

    def _initialize(self, args):
        self._load_config(args)
        self._set_overrideable_parameters(args)
        self._setup_api_client(args)

    def execute(self, args):
        self._initialize(args)
        # Test to see if input is file or single
        if os.path.isfile(args.user):
            self.process_file(args.user)
        else:
            self.process_single(args.user)



from .DSRProcessor import DSRProcessor
from .InitProcessor import InitProcessor
from .DeleteProcessor import DeleteProcessor
from .OptoutProcessor  import OptoutProcessor
from .PingProcessor  import PingProcessor
from .HashProcessor import HashProcessor
from .UserFormatError import UserFormatError
from .Request import Request
from .ApiClient import ApiClient

from .__version__ import __version__




import os
import re
import setuptools

envstring = lambda var: os.environ.get(var) or ""

here = os.path.abspath(os.path.dirname(__file__))
about = {}

with open(os.path.join(here, 'li_privacy', '__version__.py'), 'r') as f:
    exec(f.read(), about)

with open(os.path.join(here, "README.md"), "r") as fh:
    long_description = fh.read()

with open("requirements.txt", "r") as fh:
    dependencies = fh.read().strip().split("\n")

setuptools.setup(
    name=about['__title__'],
    version=about['__version__'],
    author=about['__author__'],
    author_email=about['__author_email__'],
    entry_points={"console_scripts": ["li-privacy=li_privacy.cli:main"]},
    description=about['__description__'],
    install_requires=dependencies,
    python_requires=">=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*",
    url=about['__url__'],
    long_description=long_description,
    long_description_content_type="text/markdown",
    license=about['__license__'],
    packages=[about['__title__']],
    classifiers=[
        "Operating System :: POSIX :: Linux",
        "License :: OSI Approved :: MIT License",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
)
